/*
**	Copyright (c) 1998, 1999 by Joerg Czeranski
**	All rights reserved.
**
**	Redistribution and use in source and binary forms, with or without
**	modification, are permitted provided that the following conditions are
**	met:
**
**	1. Redistributions of source code must retain the above copyright
**	   notice, this list of conditions and the following disclaimer.
**	2. Redistributions in binary form must reproduce the above copyright
**	   notice, this list of conditions and the following disclaimer in the
**	   documentation and/or other materials provided with the distribution.
**	3. The name of the author may not be used to endorse or promote
**	   products derived from this software without specific prior written
**	   permission.
**
**	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
**	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
**	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
**	DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
**	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
**	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
**	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
**	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
**	STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
**	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
**	POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xmu/WinUtil.h>

#ifndef lint
static char sccsid[] = "@(#)xdestroy.c	1.2 2/8/99";
#endif


int main(int argc, char **argv)
{
	unsigned long id;
	Window w;
	char *name, *endP;
	Display *display;

	if (argc != 2)
	{
		fprintf(stderr, "usage: %s {XID | name}\n", argv[0]);
		exit(10);
	}
	name = argv[1];

	display = XOpenDisplay(NULL);
	if (display == NULL)
	{
		fprintf(stderr, "%s: can't open display %s\n",
			argv[0], XDisplayName(NULL));
		exit(1);
	}

	if ((id = strtoul(name, &endP, 0)) == 0 || *endP)
	{
		/* not an XID */
		Window dummy_w, *children;
		unsigned n;

		w = None;

		if (XQueryTree(display, DefaultRootWindow(display), &dummy_w,
			&dummy_w, &children, &n))
		{
			int i;

			for (i = 0; i < n; i++)
			{
				Window client_w;
				char *client_name;

				client_w = XmuClientWindow(display,
					children[i]);
				if (XFetchName(display, client_w,
					&client_name) &&
					!strcmp(name, client_name))
				{
					w = client_w;
					break;
				}
			}

			if (n)
				XFree(children);
		}

		if (w == None)
		{
			fprintf(stderr, "%s: no client `%s'\n", argv[0], name);
			exit(1);
		}
	}
	else
		w = id;

	XDestroyWindow(display, w);
	XCloseDisplay(display);
	return 0;
}
